#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <utility>
#include <signal.h>
using namespace std;

int done(double res, const char * msg) {
	cout << res << '\n';
	cerr << msg << '\n';
	return 0;
}

// maintains a set of answers still possible given the interations so far
struct Options {
	vector<pair<int, int>> list;

	Options() {
		// nothing here
	}

	Options(int n, int b1, int b2) {
		if (b2 != 0) {
			list.push_back({b1, b2});
		} else if (b1 != 0) {
			for (int b2 = b1 + 1; b2 <= n; ++b2) {
				list.push_back({b1, b2});
			}
		} else if (n * (n - 1) < 2e6) {
			for (int b1 = 1; b1 < n; ++b1) {
				for (int b2 = b1 + 1; b2 <= n; ++b2) {
					list.push_back({b1, b2});
				}
			}
		} else {
			srand(n);
			set<pair<int, int>> tmp;
			while (tmp.size() < 1e6) {
				int b1 = 1 + rand() % (n - 1);
				int b2 = 1 + b1 + rand() % (n - b1);
				tmp.insert({b1, b2});
			}
			list.assign(tmp.begin(), tmp.end());
		}
	}

	// splits the set of answers depending on whether
	// the response to the query would be EXP or SOL
	pair<Options, Options> query(const set<int> & q) {
		Options exp, sol;
		for (const auto & p : list) {
			if (q.contains(p.first) and q.contains(p.second)) {
				exp.list.push_back(p);
			} else {
				sol.list.push_back(p);
			}
		}
		return {exp, sol};
	}

	// checks whether b1, b2 is the only remaining valid answer
	bool answer(int b1, int b2) {
		return list.size() == 1 and list.front() == make_pair(b1, b2);
	}
};

// set of strategies the grader may use
struct Strategy {
	virtual bool query(const set<int> & q);
	virtual bool answer(int b1, int b2);
};

struct FixedStrategy : Strategy {
	int b1, b2;
	FixedStrategy(int b1, int b2) : b1(b1), b2(b2) {
		// nothing here
	}
	bool query(const set<int> & q) override {
		return q.contains(b1) and q.contains(b2);
	}
	bool answer(int b1, int b2) override {
		return this->b1 == b1 and this->b2 == b2;
	}
};

struct MinInfoStrategy : Strategy {
	Options opt;
	MinInfoStrategy(int n, int b1, int b2) : opt(n, b1, b2) {
		// nothing here
	}
	bool query(const set<int> & q) override {
		auto [exp, sol] = opt.query(q);
		if (exp.list.size() > sol.list.size()) {
			opt = exp;
			return true;
		} else {
			opt = sol;
			return false;
		}
	}
	bool answer(int b1, int b2) override {
		return opt.answer(b1, b2);
	}
};

struct DelayExpStrategy : Strategy {
	Options opt;
	DelayExpStrategy(int n, int b1, int b2) : opt(n, b1, b2) {
		// nothing here
	}
	bool query(const set<int> & q) override {
		auto [exp, sol] = opt.query(q);
		if (sol.list.size() == 0) {
			opt = exp;
			return true;
		} else {
			opt = sol;
			return false;
		}
	}
	bool answer(int b1, int b2) override {
		return opt.answer(b1, b2);
	}
};

int main(int argc, char *argv[]) {
	signal(SIGPIPE, SIG_IGN);
	ofstream to_sol(argv[2]);
	ifstream from_sol(argv[1]);

	int n, b1, b2; string s;
	cin >> n >> b1 >> b2 >> s;
#if DEBUG > 0
	cerr << n << " " << b1 << " " << b2 << " " << s << "\n";
#endif
	Strategy *strat = new FixedStrategy(b1, b2);
	if (s == "minInfo") strat = new MinInfoStrategy(n, b1, b2);
	if (s == "delayExp") strat = new DelayExpStrategy(n, b1, b2);

#if DEBUG > 0
	cerr << "--> " << n << "\n";
#endif
	to_sol << n << endl;
	if (!to_sol) {
		return done(0, "Protocol error");
	}
	for (int queries = 0; ; ++queries) {
		char buf[7 * 100000];
		from_sol.getline(buf, sizeof(buf));
		if (!from_sol) {
			return done(0, "Protocol error");
		}
#if DEBUG > 1
	cerr << "<-- " << buf << "\n";
#endif
		istringstream is(buf);
		char q; is >> q;
		if (!is) {
			return done(0, "Protocol error");
		}
		if (q == '?') { // query
#if DEBUG == 1
			cerr << "<-- ? (" << queries << ")\n";
#endif
			if (queries == 100) {
#if DEBUG > 0
				cerr << "--> " << "ERR" << "\n";
#endif
				to_sol << "ERR" << endl;
				return done(0, "Protocol error");
			}
			set<int> q;
			int a, b = 1;
			while (is >> a) {
				if (b > a or a > n) {
#if DEBUG > 0
					cerr << "--> " << "ERR" << "\n";
#endif
					to_sol << "ERR" << endl;
					return done(0, "Protocol error");
				}
				q.insert(a);
				b = a + 1;
			}
			if (!is.eof()) {
#if DEBUG > 0
				cerr << "--> " << "ERR" << "\n";
#endif
				to_sol << "ERR" << endl;
				return done(0, "Protocol error");
			}
			auto res = strat->query(q) ? "EXP" : "SOL";
#if DEBUG > 0
			cerr << "--> " << res << "\n";
#endif
			to_sol << res << endl;
			if (!to_sol) {
				return done(0, "Protocol error");
			}
		} else if (q == '!') { // answer
#if DEBUG == 1
			cerr << "<-- !\n";
#endif
			int b1, b2; is >> b1 >> b2;
			if (!is or 1 > b1 or b1 >= b2 or b2 > n) {
#if DEBUG > 0
				cerr << "--> " << "ERR" << "\n";
#endif
				to_sol << "ERR" << endl;
				return done(0, "Protocol error");
			}
			if (is >> q) {
#if DEBUG > 0
				cerr << "--> " << "ERR" << "\n";
#endif
				to_sol << "ERR" << endl;
				return done(0, "Protocol error");
			}
			if (!strat->answer(b1, b2)) {
				return done(0, "Wrong answer");
			}
			return done(1, "OK");
		} else { // invalid
#if DEBUG > 0
			cerr << "--> " << "ERR" << "\n";
#endif
			to_sol << "ERR" << endl;
			return done(0, "Protocol error");
		}
	}
}
